@@ -18,7 +18,7 @@ from .details import *
 from .memory import *
 from .modules import *
 from capstone import *
-from capstone.x86_const import *
+from capstone.x86 import *
 
 syscall_functions = {}
 
@@ -139,7 +139,7 @@ class Dumpulator(Architecture):
                 protect = region.protect
                 if region.state == MemoryState.MEM_RESERVE:
                     protect = region.allocation_protect
-                entry[3] = protect.name
+                entry[3] = str(protect)
                 if isinstance(region.info, Module):
                     module: Module = region.info
                     entry[4] = f" {module.name}[{hex(module.size)}]"
@@ -191,7 +191,10 @@ class Dumpulator(Architecture):
     def _setup_memory(self):
         info: minidump.MinidumpMemoryInfo
         regions: List[List[minidump.MinidumpMemoryInfo]] = []
+        mask = 0xFFFFFFFFFFFFFFFF if self._x64 else 0xFFFFFFFF
         for info in self._minidump.memory_info.infos:
+            info.AllocationBase &= mask
+            info.BaseAddress &= mask
             if len(regions) == 0 or info.AllocationBase != regions[-1][0].AllocationBase or info.State == minidump.MemoryState.MEM_FREE:
                 regions.append([])
             regions[-1].append(info)
@@ -521,7 +524,7 @@ class Dumpulator(Architecture):
         self.exception.handling = True
 
         if self.exception.type == ExceptionType.ContextSwitch:
-            self.info(f"switching context, cip: {self.regs.cip}")
+            self.info(f"switching context, cip: {hex(self.regs.cip)}")
             # Clear the pending exception
             self.last_exception = self.exception
             self.exception = ExceptionInfo()
@@ -723,7 +726,7 @@ rsp in KiUserExceptionDispatcher:
     def NtCurrentThread(self):
         return 0xFFFFFFFFFFFFFFFE if self._x64 else 0xFFFFFFFE
 
-    def map_module(self, file_data: bytes, file_path: str = "", requested_base: int = 0):
+    def map_module(self, file_data: bytes, file_path: str = "", requested_base: int = 0, resolve_imports = True):
         if not file_path:
             file_path = "<unnamed>"
         print(f"Mapping module {file_path}")
@@ -765,7 +768,7 @@ rsp in KiUserExceptionDispatcher:
                     assert hint_rva is not None, "hint_rva is 0"
                     if hint_rva & ordinal_flag:
                         ordinal = f"#{hint_rva & 0xffff}"
-                        export = dll.find_export(ordinal)
+                        export = self.modules.resolve_export(dll.name, ordinal)
                         assert export is not None, f"Ordinal #{ordinal} not in {dll_name}"
                         imp_va = export.address
                         self.info(f"\t#{ordinal} = {hex(imp_va)}")
@@ -773,7 +776,7 @@ rsp in KiUserExceptionDispatcher:
                         hint = pe.get_word_from_data(pe.get_data(hint_rva, 2), 0)
                         func_name = pe.get_string_at_rva(ilt[idx].AddressOfData + 2, MAX_IMPORT_NAME_LENGTH)
                         func_name = func_name.decode("utf-8")
-                        export = dll.find_export(func_name)
+                        export = self.modules.resolve_export(dll.name, func_name)
                         assert export is not None, f"Export {func_name} not in {dll_name}"
                         imp_va = export.address
                         self.info(f"\t{func_name} = {hex(imp_va)}")
@@ -813,7 +816,7 @@ rsp in KiUserExceptionDispatcher:
                 protect = MemoryProtect.PAGE_READWRITE
             if execute:
                 protect = MemoryProtect(protect.value << 4)
-            print(f"Mapping section '{name.decode()}' {hex(rva)}[{hex(rva)}] -> {hex(va)} as {protect.name}")
+            print(f"Mapping section '{name.decode()}' {hex(rva)}[{hex(rva)}] -> {hex(va)} as {protect}")
             self.memory.commit(va, size, protect)
             self.write(va, data)
 
@@ -855,9 +858,13 @@ def _hook_code_exception(uc: Uc, address, size, dp: Dumpulator):
         raise err
 
 def _hook_mem(uc: Uc, access, address, size, value, dp: Dumpulator):
+    fetch_accesses = [UC_MEM_FETCH, UC_MEM_FETCH_PROT, UC_MEM_FETCH_UNMAPPED]
     if access == UC_MEM_FETCH_UNMAPPED and address >= FORCE_KILL_ADDR - 0x10 and address <= FORCE_KILL_ADDR + 0x10 and dp.kill_me is not None:
         dp.error(f"forced exit memory operation {access} of {address:x}[{size:x}] = {value:X}")
         return False
+    if dp.exception.final and access in fetch_accesses:
+        dp.info(f"fetch from {hex(address)}[{size}] already reported")
+        return False
     # TODO: figure out why when you start executing at 0 this callback is triggered more than once
     try:
         # Extract exception information
@@ -868,10 +875,11 @@ def _hook_mem(uc: Uc, access, address, size, value, dp: Dumpulator):
         exception.memory_size = size
         exception.memory_value = value
         exception.context = uc.context_save()
-        tb = uc.ctl_request_cache(dp.regs.cip)
-        exception.tb_start = tb.pc
-        exception.tb_size = tb.size
-        exception.tb_icount = tb.icount
+        if access not in fetch_accesses:
+            tb = uc.ctl_request_cache(dp.regs.cip)
+            exception.tb_start = tb.pc
+            exception.tb_size = tb.size
+            exception.tb_icount = tb.icount
 
         # Print exception info
         final = dp.trace or dp.exception.code_hook_h is not None
@@ -922,7 +930,8 @@ def _hook_mem(uc: Uc, access, address, size, value, dp: Dumpulator):
 
         # Remove the translation block cache for this block
         # Without doing this single stepping the block won't work
-        uc.ctl_remove_cache(exception.tb_start, exception.tb_start + exception.tb_size)
+        if exception.tb_start != 0:
+            uc.ctl_remove_cache(exception.tb_start, exception.tb_start + exception.tb_size)
 
         # Install the code hook to single step the basic block again.
         # This will prevent translation block caching and give us the correct cip
@@ -1059,6 +1068,7 @@ def _hook_interrupt(uc: Uc, number, dp: Dumpulator):
         exception.type = ExceptionType.Interrupt
         exception.interrupt_number = number
         exception.context = uc.context_save()
+        # TODO: this might crash if cip is not valid memory
         tb = uc.ctl_request_cache(dp.regs.cip)
         exception.tb_start = tb.pc
         exception.tb_size = tb.size
@@ -1090,6 +1100,9 @@ def _hook_interrupt(uc: Uc, number, dp: Dumpulator):
     raise UcError(UC_ERR_EXCEPTION)
 
 def _hook_syscall(uc: Uc, dp: Dumpulator):
+    # Flush the trace for easier debugging
+    if dp.trace is not None:
+        dp.trace.flush()
     index = dp.regs.cax & 0xffff
     if index < len(dp.syscalls):
         name, syscall_impl, argcount = dp.syscalls[index]
@@ -1111,7 +1124,7 @@ def _hook_syscall(uc: Uc, dp: Dumpulator):
                     argvalue = argtype(argvalue, dp)
                 elif issubclass(argtype, Enum):
                     try:
-                        argvalue = argtype(dp.args[i])
+                        argvalue = argtype(dp.args[i] & 0xFFFFFFFF)
                     except KeyError as x:
                         raise Exception(f"Unknown enum value {dp.args[i]} for {type(argtype)}")
                 else:
@@ -1149,6 +1162,21 @@ def _hook_syscall(uc: Uc, dp: Dumpulator):
         dp.error(f"syscall index {index:x} out of range")
         dp.raise_kill(IndexError())
 
+def _emulate_unsupported_instruction(dp: Dumpulator, instr: CsInsn):
+    if instr.id == X86_INS_RDRAND:
+        op: X86Op = instr.operands[0]
+        regname = instr.reg_name(op.reg)
+        if dp._x64 and op.size * 8 == 32:
+            regname = "r" + regname[1:]
+        print(f"emulated rdrand {regname}:{op.size * 8}, cip = {hex(instr.address)}+{instr.size}")
+        dp.regs[regname] = 42  # TODO: PRNG based on dmp hash
+        dp.regs.cip += instr.size
+    else:
+        # Unsupported instruction
+        return False
+    # Resume execution
+    return True
+
 def _hook_invalid(uc: Uc, dp: Dumpulator):
     address = dp.regs.cip
     # HACK: unicorn cannot gracefully exit in all contexts
@@ -1156,4 +1184,20 @@ def _hook_invalid(uc: Uc, dp: Dumpulator):
         dp.error(f"terminating emulation...")
         return False
     dp.error(f"invalid instruction at {address:x}")
+    try:
+        code = dp.read(address, 15)
+        instr = next(dp.cs.disasm(code, address, 1))
+        if _emulate_unsupported_instruction(dp, instr):
+            # Resume execution with a context switch
+            assert dp.exception.type == ExceptionType.NoException
+            exception = ExceptionInfo()
+            exception.type = ExceptionType.ContextSwitch
+            exception.final = True
+            dp.exception = exception
+            return False  # NOTE: returning True would stop emulation
+    except StopIteration:
+        pass  # Unsupported instruction
+    except UcError:
+        pass  # Invalid memory access (NOTE: this should not be possible actually)
+    raise NotImplementedError("TODO: throw invalid instruction exception")
     return False
