@@ -18,6 +18,9 @@ STATUS_SET_CONTEXT_DENIED = 0xC000060A  # Return from NtContinue to int 29
 STATUS_INFO_LENGTH_MISMATCH = 0xC0000004
 STATUS_INVALID_PARAMETER = 0xC000000D
 STATUS_OBJECT_NAME_NOT_FOUND = 0xC0000034
+STATUS_NOT_FOUND = 0xC0000225
+STATUS_MEMORY_NOT_ALLOCATED = 0xC00000A0
+STATUS_CONFLICTING_ADDRESSES = 0xC0000018
 
 # Exceptions
 DBG_PRINTEXCEPTION_C = 0x40010006
